from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class DynamodbRecovery(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Dynamodb point in time recovery (backup) is enabled"
        id = "CKV_AWS_28"
        supported_resources = ['AWS::DynamoDB::Table']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for Point in Time Recovery for DynamoDB Table:
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html
        :param conf: ddb_table configuration
        :return: <CheckResult>
        """
        if 'Properties' in conf.keys():
            if 'PointInTimeRecoverySpecification' in conf['Properties'].keys():
                if 'PointInTimeRecoveryEnabled' in conf['Properties']['PointInTimeRecoverySpecification'].keys():
                    if conf['Properties']['PointInTimeRecoverySpecification']['PointInTimeRecoveryEnabled'] == True:
                        return CheckResult.PASSED
        return CheckResult.FAILED

check = DynamodbRecovery()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.consts import ANY_VALUE


class GoogleSubnetworkLoggingEnabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that VPC Flow Logs is enabled for every subnet in a VPC Network"
        id = "CKV_GCP_26"
        supported_resources = ['google_compute_subnetwork']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'log_config'

    def get_expected_values(self):
        return [ANY_VALUE]


check = GoogleSubnetworkLoggingEnabled()



from checkov.terraform.checks.resource.azure.NSGRulePortAccessRestricted import NSGRulePortAccessRestricted


class NSGRuleRDPAccessRestricted(NSGRulePortAccessRestricted):
    def __init__(self):
        super().__init__(name = "Ensure that RDP access is restricted from the internet",check_id = "CKV_AZURE_9",port=3389)


check = NSGRuleRDPAccessRestricted()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
import re

PORT_RANGE = re.compile('\d+-\d+')


class NSGRulePortAccessRestricted(BaseResourceCheck):
    def __init__(self, name, check_id, port):
        supported_resources = ['azure_security_group_rule', 'azurerm_network_security_rule']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def is_port_in_range(self, conf):
        ports = force_list(conf['destination_port_range'][0])
        for range in ports:
            if re.match(PORT_RANGE, range):
                start, end = int(range.split('-')[0]), int(range.split('-')[1])
                if start <= self.port <= end:
                    return True
            if range in [str(self.port), '*']:
                return True
        return False

    def scan_resource_conf(self, conf):
        if 'access' in conf and conf['access'][0] == "Allow":
            if 'direction' in conf and conf['direction'][0] == "Inbound":
                if 'protocol' in conf and conf['protocol'][0] == 'TCP':
                    if 'destination_port_range' in conf and self.is_port_in_range(conf):
                        if 'source_address_prefix' in conf and conf['source_address_prefix'][0] in ["*", "0.0.0.0",
                                                                                                    "<nw>/0", "/0",
                                                                                                    "internet", "any"]:
                            return CheckResult.FAILED
        return CheckResult.PASSED

